'--------------------------------------------------------------------------------------------------
' Created by:  Valdis Vitolins
' Purpose: common library for QTP functions
' Comments: This library should be for test settings: Test-Settings...-Resources-Libraries and add Functions.vbs file
' Edit this file with some user-friendly editor, which shows line numbers (necessary to debug QTP errors), e.g. VbsEdit
'--------------------------------------------------------------------------------------------------
Option Explicit 							' declare variables explicitly to distinct global and local variables
Public Const cDATA_DIR = "Data"				' Subfolder name for test data
public const cTOOLS_DIR = "Tools"			' Subfolder for tools
public const cLOG_DIR = "Results"			' Subfolder name for run logs 
Public Const cBrowser = "C:\Program Files\Internet Explorer\iexplore.exe" ' path to Ienternet Explorer executable file
public const cDATA_FILE="Data.xls"			' Common test data file

Public const cPASS = 0		' micPass: Causes the status of this step to be passed and sends the specified message to the report. 
Public const cFAIL = 1 		' micFail: Causes the status of this step to be failed and sends the specified message to the report. When this step runs, the test or component fails. 
Public const cDONE = 2		' micDone: Sends a message to the report without affecting the pass/fail status of the test or component. 
Public Const cWARNING = 3	' micWarning: Sends a warning message to the report, but does not cause the test or component to stop running, and does not affect the pass/fail status of the test or component.

' Declare some variables explicitly, to be sure they exist before invocation of init() function
' Otherwise, for variables declared in InvokeFunctions list, such declaration is not necessary

Public root_dir									' Root directory for scripts. It is used to get full path 
Public run_label								' label for test logs (*.log file name, value for Run field in Log table)
Public url										' URL to the application index page
Public username									' default application username 
Public password									' default application password
Public log_file									' runlog file name
Public timeout									' timeout for waiting operations
Public test_status								' the worst status for current test, is set by report_log()
Public funct									' name of invoked function (value is set in InvokeFunctions action)
Public crow										' current row of InvokeFunctions sheet, which is executed in by Batch test action
Public debug_on									' Triggering this variable to True/False will show/hide message boxes with commad lines before they execution

'--------------------------------------------------------------------------------------------------
'@Description global variables are declared in Functions.vbs script usually this function is not called dirtectly but is called by Start action instead
'@Documentation initialize values for global variables as code shortcuts in place of global data parameters
Public Function init()
	Dim quote,variable,val,i,j,rowcount,msg,msga
	quote = chr(34)
	root_dir = Environment("TestDir")
	root_dir = Left(root_dir,InStrRev(root_dir,"\")-1)
	If  root_dir = "" Then 											' stop run, If shared vbs library is not added
			Reporter.ReportEvent 1, "Init", "Init constants are empty. Check that Test-Settings...-Resources-Libraries have added Functions.vbs file"
			exitrun(1)
	End If
	rowcount = DataTable.GetSheet("Global").GetRowCount
	If  rowcount = 0 Then 												' stop run, If common data file is not added
			Reporter.ReportEvent 1, "init", "Variable definition is empty. Check that Test-Settings...-Resources-Data Table-Other Location points to " & cDATA_FILE & " file"
			exitrun(1)
	End If
	For i = 1 to rowcount
		DataTable.SetCurrentRow (i) 
		variable = DataTable("Variable", "Global")
		val = DataTable("Value", "Global")
		If variable<>"" Then
			If Not IsNumeric(val) Then val = quote & val & quote
			ExecuteGlobal (variable & " = " & val)
			msg=msg&variable&" = "&val&vbnewline
		End If
	Next
	If run_label="" Then run_label = Year(Date())&"-"&Month(Date())&"-"&Day(Date())		' set runlog label
	log_file = root_dir&"\"&cLOG_DIR&"\"&run_label&".log"
	msga=split(msg,vbnewline)
	For i=0 to ubound(msga)-1
		report_log cPASS,msga(i),msga(i)  	' log variable values	
	Next
	set_obj_id_settings						' set object settings to recognize them in the same manner
End Function

'--------------------------------------------------------------------------------------------------
'@Description Open application window, it is not opened already
'@Documentation Open application window, it is not opened already
Public Function run_app()
	Dim msg
	If not Browser("My Application").Exist(1) Then
		SystemUtil.Run cBrowser,url 
		report_log cDONE,"run_app","Application is started" 
	End If
	With Browser("My Application")
		If .Page("My Application Login Page").Exist(1) Then
			report_log cDONE,"run_app","Application window is opened"
			Exit function
		end if 
		If .Page("My Application").Exist(1) Then				' Start application, If it is not opened
			report_log cPASS,"run_app","Authentication window is opened"  
		else
			report_log cFAIL,"run_app","Authentication window is not opened"  
			exitrun cFAIL
		end if
	End With
End Function

'--------------------------------------------------------------------------------------------------
'@Description Close application window
'@Documentation Close application window
Public Function close_app()
	With Browser("My Application")
		If .Exist(1) Then				
			.Close
			wait 1
			If Not .Exist(1) Then
				report_log cDONE,"close_app","Application is closed successfully"   								
			else
				report_log cWARNING,"close_app","Application is not closed"   								
			End If
		Else
			report_log cWARNING,"close_app","Application window doesn't exist"   								
		End If
	End With 
End Function

'--------------------------------------------------------------------------------------------------
'@Description Handy wrapper for application management
'@Documentation Initialize environemnt and start application
Public Function start()
	init												' initialize environment
	run_app												' start application
	logon "",""											' logon With default username and password
	If Browser("My Application").Page("My Application").Exist(1) then 
		Browser("My Application").Page("My Application").Activate
		Browser("My Application").Page("My Application").Resize app_width,app_height		' set application window size and location
		Browser("My Application").Page("My Application").Move app_corner_x,app_corner_y
	end If
	
End Function 

'--------------------------------------------------------------------------------------------------
'@Description log on into application
'@Documentation log on into application
Public Function logon(lusername, lpassword)
	Dim cuser,msg
	If lusername="" Then lusername=username				' use global parameters, If local aren't passed
	If lpassword="" Then lpassword=password
	With Browser("My Application")
		If .Page("My Application").Exist(1) Then					' Check for application window
			get_username cuser
			If cuser=lusername Then
				report_log cDONE,"Logon","Have logeed on in application as:"&lusername
			Else 
				report_log cWARNING,"Logon","Have logged on in application as:"&cuser&". Will log off and login again"
				.Page("My Application").WebButton("Logoff").Click
				.Page("My Application Login Page").Sync
			End If
		else
			report_log cWARNING,"Logon","Application window is not opened. Will open"
			run_app 										' open application
		end if							
						' after application is started
	   If .Page("My Application Login Page").Exist(1) then	
			report_log cDONE,"Start","Application Authentication window appeared"
	
			.Page("My Application Login Page").WebEdit("username").Set lusername
			.Page("My Application Login Page").WebEdit("password").Set lpassword
			.Page("My Application Login Page").WebButton("Login").Click
			.Page("My Application").Sync
	
			If .Page("My Application").Exist(timeout) then
				report_log cDONE, "Logon OK", "Application window appeared"
				logon = cPASS
			else
				report_log cFAIL, "Logon error", "Application window didn't appear"
				logon = cFAIL
			end If
		end If
	End With
End Function

'--------------------------------------------------------------------------------------------------
'@Description log off from the application
'@Documentation log off from the application
Public Function logoff ()
	With Browser("My Application")
		.Page("My Application").WebButton("Logoff").Click
		If .Page("My Application Login Page").exist(1) Then
			report_log cPASS, "logoff", "Login Page appeared successfully"
		Else
			report_log cFAIL, "logoff", "Login Page didn't appear"
		End If 
	End With
End Function

'--------------------------------------------------------------------------------------------------
'@Description go to the row of InvokeFunctions sheet, which is marked with passed label
'@Documentation go to the row, which is marked with <label>
Function golabel (label)
	Dim rowcount,i,cond,tmprow
	tmprow=crow
	rowcount = DataTable.GetSheet("InvokeFunctions").GetRowCount	' for all rows in table
	With DataTable
		For i = 1 to rowcount
			DataTable.SetCurrentRow (i) 							' load parameters from table
			cond = DataTable("Condition", "InvokeFunctions")
			If strcomp(cond,label,vbTextCompare)=0 Then
				crow=i-1
				Exit Function
			End If
		Next
		If i>rowcount Then											' if label is not found, log and go to the next row	
			report_log cWARNING, "golabel", "Label '"&label&"' is not found. Skipping to the next row ("&tmprow+1&")"
			crow=tmprow
		End If
	End with
End Function

'--------------------------------------------------------------------------------------------------
'@Description report log in QTP log and text file
'@Documentation report log in QTP log and text file
Public function report_log(status, step_name, step_description)
	On Error Resume Next	' skip writigt to the log file, If path is not initialized
	Dim test_log
	Reporter.ReportEvent status,  step_name, step_description
	test_log = status & vbTab & step_name & vbTab & step_description & vbTab & funct & vbTab & Now()
	file_append log_file, test_log
	If status=cFAIL Then test_status=cFAIL	' fail test status only If there is error
End function

'--------------------------------------------------------------------------------------------------
'@Description append line to the file
'@Documentation append line to the <file_name>
Public function file_append(file_name, svalue)
	Dim fso,DirFilet 
	Set fso = CreateObject("Scripting.FileSystemObject")
	Set DirFilet = fso.OpenTextFile(file_name, 8, True)
	DirFilet.WriteLine svalue
	DirFilet.Close   ' Close Dir found files
end Function

'--------------------------------------------------------------------------------------------------
'@Description get message from the messagebox and close it
'@Documentation Handle messagebox
Public function get_message(ByRef message, button, timeout)
	On Error Resume Next
	If button = "" Then button = "OK" 	' set default button name for checking performance
	If timeout = "" Then timeout = 1 	' set default timeout time
	With Dialog("MessageBox")
		If .Exist(timeout) Then
			message=.GetVisibleText
			message = mid(message,1,InStrRev(message, vbNewLine)-1)
			message = replace(message, vbNewLine, ": ")
			If Not .WinButton(button).Exist(1) then
            	report_log cWARNING, "get_message", "Messagebox '" & message & "' doesn't contain button '" & button &"'"
            	If .WinButton("OK").Exist(1) then  button = "OK"
				If .WinButton("Yes").Exist(1) then button = "Yes"
				If .WinButton("Cancel").Exist(1) then button = "Cancel"
			End If 
			.WinButton(button).Click
			report_log cDONE, "get_message", "Button '" & button & "' is pressed for message '" & message & "'"
			get_message = cPASS
		Else
			get_message = cFAIL
		End If
	End with
End Function

'--------------------------------------------------------------------------------------------------
'@Description Return value from the cell of given table
'@Documentation Return value from <table_obj> (<row>,<col>)
public function tbl_get_cell_value(ByRef table_obj, ByVal row, ByVal col, ByRef retvalue) 
	On error resume Next
	Dim table,rows_count,src
	Set table = table_obj.Object
	With table
		rows_count = .rowcount 				' check count of rows
		If row>rows_count then
			report_log cWARNING, "tbl_get_cell_value", "Passed row '"&row&"' is out of count '"&rows_count&"'"
			tbl_get_cell_value = cFAIL
			Exit Function
		End If
        cols_count = .colcount				' check count of columns
		If (col>cols_count) Then
			report_log cWARNING, "tbl_get_cell_value", "Passed column '"&col&"' is out of count '"&cols_count&"'"
			tbl_get_cell_value = cFAIL
			Exit Function
		End If
	End with
	With table.item(col,row)
		select case .CellType
		Case "ComboBox"
			Set src=.DataSource				' these two lines may cause error, If combo box is empty
			retvalue=src.item(.Text).row.item(1)
		Case else
			retvalue=.Text
		end select
		tbl_get_cell_value = cPASS
	end with
end function



'--------------------------------------------------------------------------------------------------
'@Description returns name of the current application user
'@Documentation returns name of the current application user
Public function get_username(Byref user)
	With Browser("My Application").Page("My Application").WebElement("Hello")
		If .exist(1) Then 
			user = .GetROProperty("innertext")
			user = Mid(user,7)
			report_log cDONE, "get_username","Current user is:'"&user&"'"	
		End If
	End With
End Function

'--------------------------------------------------------------------------------------------------
'@Description object identification setting are set under Tools-Object Identification... and stored in Tools/ObjectIdentificationSettings.vbs 
'@Documentation apply object identification settings dynamically
Public function set_obj_id_settings()
	'On error resume next
	Dim App 'As Application
	Set App = CreateObject("QuickTest.Application")
	App.Launch
	App.Visible = True
	' as it goes very slowly, only changes from default settings are reset

	'Configuration of Standard Windows objects
	
	'Object identification configuration for test object object "dialog"
	App.Options.ObjectIdentification("dialog").MandatoryProperties.RemoveAll()
	App.Options.ObjectIdentification("dialog").AssistiveProperties.RemoveAll()
	App.Options.ObjectIdentification("dialog").MandatoryProperties.Add("nativeclass")
	App.Options.ObjectIdentification("dialog").MandatoryProperties.Add("text")
	App.Options.ObjectIdentification("dialog").EnableSmartIdentification = False
	App.Options.ObjectIdentification("dialog").BaseFilterProperties.RemoveAll()
	App.Options.ObjectIdentification("dialog").OptionalFilterProperties.RemoveAll()
	
	'Object identification configuration for test object object "window"
	App.Options.ObjectIdentification("window").MandatoryProperties.RemoveAll()
	App.Options.ObjectIdentification("window").AssistiveProperties.RemoveAll()
	App.Options.ObjectIdentification("window").MandatoryProperties.Add("regexpwndclass")
	App.Options.ObjectIdentification("window").AssistiveProperties.Add("text")
	App.Options.ObjectIdentification("window").BaseFilterProperties.RemoveAll()
	App.Options.ObjectIdentification("window").OptionalFilterProperties.RemoveAll()

	'Object identification configuration for test object object "page"
	App.Options.ObjectIdentification("page").MandatoryProperties.RemoveAll()
	App.Options.ObjectIdentification("page").AssistiveProperties.RemoveAll()
	App.Options.ObjectIdentification("page").MandatoryProperties.Add("title")
	App.Options.ObjectIdentification("page").BaseFilterProperties.RemoveAll()
	App.Options.ObjectIdentification("page").OptionalFilterProperties.RemoveAll()
	report_log cDONE, "set_obj_id_settings", "ObjectIdentificationSettings are applied successfully"
End Function

'--------------------------------------------------------------------------------------------------
'@Description Test and check data submit in application
'@Documentation Test and check data submit in application
Public Sub Test1(val1,val2)
	Dim msg
	With Browser("My Application").Page("My Application")
		.WebList("Numbers").Select "Two"
		Browser("My Application").Page("My Application").WebButton("Add").Click
		.WebButton("Add").Click
		.WebCheckBox("chkValidateSubmit").Set "ON"
		.WebEdit("txtRow1").Set val1
		.WebList("selRow0").Select "value1"
		.WebEdit("txtRow2").Set val2
		.WebList("selRow2").Select "value1"
		.WebButton("Remove").Click
		.WebButton("Submit").Click
		get_message msg, "OK", 1
		If msg<>"" Then
			report_log cFAIL, "Test1", "Validation error '"&msg&"' apeared"
			Exit Sub
		End If
		Browser("Submit results").Page("Submit results").Sync
		msg=Browser("Submit results").Page("Submit results").WebElement("Numbers").GetTOProperty("innertext")
		If instr(msg,val1)>0 and instr(msg,val2)>0 Then
			report_log cPASS, "Test1", "Values "&val1&" and "&val2&" appeared in Submit results window"
		Else
			report_log cFAIL, "Test1", "Values "&val1&" and "&val2&" didn't appear in Submit results window"
		End If
		Browser("Submit results").Close
	End With
End Sub

